
class MissingDataException(Exception):
    """This exception should be raised, when initializing a Series object
    no data is found either in` data` or both x and y axis points"""


class MissingAxisException(Exception):
    """This exception should be raise when a Series object doesn't have some
    axis information
    """


class MultipleAxisException(Exception):
    """This exception should be raise when repeated axis are found on the same
    Series object"""


class DuplicateLabelException(Exception):
    ""


class SeriesInvalidOptionException(Exception):
    ""




"""
`PyFlot` is a Python interface for the known `Flot` Javascript chart library
"""

__version__ = "0.1"
get_version = lambda : __version__

import time
try:
    import simplejson as json
except ImportError:
    import json

import exception

class BaseOptions(dict):
    "Base implementation for options"

    allowed_options = {}

    def __setitem__(self, key, value):
        """
        __setitem__ should only allow keys within allowed option keys
        and they have to batch the type
        """
        if not key.startswith('__'):
            if key not in self.allowed_options.keys():
                raise TypeError("%s is not a allowed option. Allowed "
                        "options are: %s" % (key, self.allowed_options.keys()))
            if not isinstance(value, self.allowed_options[key]):
                raise TypeError("Option %s got an unexpected object type" % key)
        super(BaseOptions, self).__setitem__(key, value)

    def __init__(self, *args, **kwargs):
        super(BaseOptions, self).__init__()
        for option, value in kwargs.items():
            if value:
                self[option] = value


class GraphOptions(BaseOptions):
    "Option object for graph"

    allowed_options = {
                        'xaxis': dict,
                        'yaxis': dict,
                        'legend': dict,
                        'grid': dict,
    }


class SeriesOptions(BaseOptions):
    """
        Options for Series object. This should be a completition of the
    normal json_data object
    """

    allowed_options  = {
            'label': basestring,
            'color': basestring,
            'xaxis': list,
            'yaxis': list,
            'lines': dict,
            'bars': dict,
            'points': dict,
            'clickable': bool,
            'hoverable': bool,
            'shadowSize': int,
            'highlightColor': basestring,
    }


class Variable(object):
    "Genric Variable Object"

    def contribute_to_class(self, obj, attr_name, data=None):
        if getattr(obj, self._var_name, False):
            raise exception.MultipleAxisException(
                                        "%s already set" % self._var_name)
        setattr(obj, self._var_name, self)
        if data:
            self._set_points(attr_name, data)


class Axis(object):
    "Generic Axis object"

    def __repr__(self):
        return self._var_name

    def __str__(self):
        return self._var_name


class XAxis(Axis):
    "X Axis Object"
    _var_name = '_x'


class YAxis(Axis):
    "Y Axis Object"
    _var_name = '_y'


class LinearVariable(Variable):
    "Linear Variable Object. No adjustment its done to the points"
    def __init__(self, points=None, **kwargs):
        if points is not None:
            self.points = points

    def _set_points(self, attr_name, data):
        self.points = []
        for sample in data:
            try:
                self.points.append(getattr(sample, attr_name))
            except AttributeError:
                s = {}
                s['_x'], s['_y'] = sample
                self.points.append(s[self._var_name])


class TimeVariable(Variable):
    "Time Variable Object. Points content its adjusted"
    def __init__(self, points=None, **kwargs):
        if points is not None:
            self.points = [int(time.mktime(point.timetuple()) * 1000)\
                                                    for point in points]

    def _set_points(self, attr_name, data):
        an = attr_name
        self.points = [int(time.mktime(getattr(s, an).timetuple()) * 1000)\
                                                    for s in data]
        self.points = []
        for sample in data:
            try:
                self.points.append(
                    int(time.mktime(getattr(sample, an).timetuple()) * 1000)
                )
            except AttributeError:
                s = {}
                s['_x'], s['_y'] = sample
                self.points.append(
                    int(time.mktime(s[self._var_name].timetuple()) * 1000)
                )


class XVariable(XAxis, LinearVariable):
    "Linear Variable on X Axis"


class YVariable(YAxis, LinearVariable):
    "Linear Variable on Y Axis"


class TimeXVariable(XAxis, TimeVariable):
    "Time Variable on X Axis"


class TimeYVariable(YAxis, TimeVariable):
    "Time Variable on Y Axis"


class Series(dict):
    "This class represents the actual flot series"

    _options = SeriesOptions()

    def _set_data(self):
        """
        This method will be called to set Series data
        """
        if getattr(self, 'data', False) and not getattr(self, '_x', False) and not getattr(self, '_y', False):
            _x = XVariable()
            _y = YVariable()
            _x.contribute_to_class(self, 'X', self.data)
            _y.contribute_to_class(self, 'Y', self.data)
            self['data'] = zip(self._x.points, self._y.points)
        else: 
            for axis in ('_x', '_y'):
                axis_obj = getattr(self, axis, False)
                if not axis_obj:
                    raise exception.MissingAxisException("%s missing" % axis)
                if not getattr(axis_obj, 'points', False):
                    raise exception.MissingDataException()

            self['data'] = zip(self._x.points, self._y.points)

    def __init__(self, data=None, xpoints=None, ypoints=None, options=None, *args, **kwargs):
        """
        Series should be able to get contructed and completed through
        class definition or through kwargs
        """
        #FIXME
        #   There should be a sanitizing method to take care of data, xpoints
        # and ypoints args. 
        self._options = SeriesOptions()
        super(Series, self).__init__()

        if 'data' not in dir(self) and data is not None:
            # if not data is defined in Class def try to get is by args
            self.data = data
        if data is None and xpoints is not None and ypoints is not None:
            # xpoints and ypoints were passed directly to Series object
            self.data = zip(xpoints, ypoints)

        # through class definition
        for attr_name in dir(self):
            attr = getattr(self, attr_name)
            if isinstance(attr, Variable):
                attr.contribute_to_class(self, attr_name, self.data)

        # through kwargs
        for name, attr in kwargs.iteritems():
            if isinstance(attr, Variable):
                attr.contribute_to_class(self, name)
        
        # set series data
        self._set_data()

        # set series options
        for option in dir(self.Meta):
            self._options[option] = getattr(self.Meta, option)
        self.update(self._options)
        # options passed to the construction will overide Meta options
        if options is not None and isinstance(options, SeriesOptions):
            self.update(options)

    @property
    def json_series(self):
        "serializes a Series object"
        return json.dumps(self)

    class Meta:
        pass


class Graph(object):
    "Contains the data object and also the plot options"

    _series = []
    _options = GraphOptions()

    def __init__(self, series=None, options=None, **kwargs):
        "This contructor will be able to receive"
        self._series = []
        # set series
        for attr_name in dir(self):
            attr = getattr(self, attr_name)
            if isinstance(attr, Series):
                self._series.append(attr)
        for arg in kwargs.values():
            if isinstance(arg, Series):
                self._series.append(arg)
        
        if series and all([isinstance(s, Series) for s in series]):
            self._series.extend(series)

        # set options through Meta
        for option in dir(self.Meta):
            self._options[option] = getattr(self.Meta, option)
        # check for options through argument
        if options and isinstance(options, GraphOptions):
            self._options.update(options)
        self._set_options()

    def _get_axis_mode(self, axis):
        "will get the axis mode for the current series"
        if all([isinstance(getattr(s, axis), TimeVariable) for s in self._series]):
            return 'time'
        return None

    def _set_options(self):
        "sets the graph ploting options"
        # this is aweful
        # FIXME: Axis options should be passed completly by a GraphOption
        if 'xaxis' in self._options.keys():
            self._options['xaxis'].update(
                        {'mode' : self._get_axis_mode(XAxis._var_name)})
        if 'yaxis' in self._options.keys():
            self._options['yaxis'].update(
                        {'mode' : self._get_axis_mode(YAxis._var_name)})

    @property
    def json_data(self):
        "returns its json data serialized"
        return json.dumps(self._series)

    @property
    def options(self):
        "returns its json option serialized"
        return json.dumps(self._options)

    class Meta:
        # meta should contain graph options, eg. xaxis, yaxis, legend
        pass




import datetime
import time
import unittest
try:
    import json
except ImportError:
    import simplejson as json

import flot
from flot import exception

class SampleObject(object):
    ""

    def __init__(self, **kwargs):
        for arg in kwargs:
            setattr(self, arg, kwargs[arg])


class S1(flot.Series):
    "Series for function y = x + 3"

    x = flot.XVariable()
    y = flot.YVariable()
    data = [SampleObject(x=i, y=i+3) for i in range(0, 10)]

    class Meta:
        label = 'series1'
        color = 'red'


class S2(flot.Series):
    "Series for function y = x"

    x = flot.XVariable()
    y = flot.YVariable()
    data = [SampleObject(x=i, y=i) for i in range(0, 10)]

    class Meta:
        label = 'series2'


class S3(flot.Series):
    "Series with no initial data"

    x = flot.XVariable()
    y = flot.YVariable()

    class Meta:
        label = 'series3'


class S4(flot.Series):
    "Series with X axis in time"

    var = flot.TimeXVariable()
    usr = flot.YVariable()
    data = [SampleObject(var=datetime.date(2011, 1, i), usr=i+10)\
                                                for i in range(1, 10)]


class OptionsTest(unittest.TestCase):
    "Test for the option global Option class"

    def test_options_check_for_allowed_options(self):
        # this object only allows
        options = flot.GraphOptions()
        key_val_list = [('x', 123), ('xaxis', [1,2,3])]
        for key, val in key_val_list:
            try:
                options[key] = val 
            except TypeError:
                # its fine
                pass
        options['xaxis'] = { 'format': '%d-%m-%y', 'mode': 'time' }

    def test_options_can_be_passed_through_constructor(self):
        options = flot.GraphOptions(xaxis={'mode': 'time'})
        self.assertTrue('xaxis' in options.keys())
        self.assertEquals(options['xaxis']['mode'], 'time')


class VariableTest(unittest.TestCase):
    "Variable test class"

    def test_receives_data(self):
        points = [x for x in range(0, 10)]
        my_var = flot.XVariable(points=points)
        self.assertEquals(my_var.points, points)


class TimeVariableTest(unittest.TestCase):
    "TimeVariable test class"

    def test_corrects_data(self):
        points = [datetime.date(2012, 1, i) for i in range(1, 10)]
        my_var = flot.TimeXVariable(points=points)
        time_points = [int(time.mktime(d.timetuple())*1000) for d in points]
        self.assertEquals(my_var.points, time_points)


class SeriesTest(unittest.TestCase):
    "Series test class"

    def test_cannot_initialize_empty_series(self):
        try:
            flot.Series(x=flot.XVariable(),
                             y=flot.YVariable())
        except exception.MissingDataException:
            pass

    def test_cannot_initialize_with_missing_axis(self):
        try:
            flot.Series(x=flot.XVariable(points=[1,2,3]))
        except exception.MissingAxisException:
            pass

    def test_cannot_duplicate_axis(self):
        try:
            flot.Series(x=flot.XVariable(points=[1,2,3]),
                    y=flot.XVariable(points=[4,5,6]))
        except exception.MultipleAxisException:
            pass

    def test_series_has_attrs(self):
        series = S1()
        self.assertEquals(series._x, S1.x)
        self.assertEquals(series._y, S1.y)
        self.assertTrue(isinstance(series._y, flot.YVariable))
        self.assertTrue(isinstance(series._x, flot.XVariable))

    def test_series_has_data(self):
        series = S1()
        y_points = [getattr(obj, 'y') for obj in series.data]
        x_points = [getattr(obj, 'x') for obj in series.data]
        self.assertEquals(series._x.points, x_points)
        self.assertEquals(series._y.points, y_points)
        self.assertEquals(series['data'], zip(x_points, y_points))

    def test_checks_meta_attrs(self):
        series = S1()
        self.assertEquals(series['label'], 'series1')
        self.assertEquals(series['color'], 'red')

    def test_series_receives_data_in_kwargs(self):
        series = S3(data=[SampleObject(x=i, y=(-4*i)) for i in range(0, 10)])
        y_data = [getattr(obj, 'y') for obj in series.data]
        x_data = [getattr(obj, 'x') for obj in series.data]
        self.assertEquals(series['data'], zip(x_data, y_data))

    def test_series_accept_field_objects_in_kwargs(self):
        x_points = [x for x in range(0, 10)]
        y_points = [y for y in range(10, 20)]
        x_field = flot.XVariable(points=x_points)
        y_field = flot.YVariable(points=y_points)
        series = flot.Series(xa=x_field, ya=y_field)
        self.assertEquals(series['data'], zip(x_points, y_points))
        self.assertEquals(series._x, x_field)
        self.assertEquals(series._y, y_field)

    def test_options_van_be_passed_to_seties(self):
        series = S1(options=flot.SeriesOptions(bars={'show': True}))
        self.assertTrue('show' in series['bars'])
        self.assertTrue(series['bars']['show'])

    def test_series_accepts_direct_data(self):
        xpoints = [1,2,3]
        ypoints = [4,5,6]
        series_data = zip(xpoints, ypoints)
        test_series = flot.Series(data=series_data)
        self.assertTrue(hasattr(test_series, '_x'))
        self.assertTrue(test_series._x.points, xpoints)
        self.assertTrue(hasattr(test_series, '_y'))
        self.assertTrue(test_series._y.points, ypoints)

    def test_series_accepts_direct_xpoints_and_ypoints(self):
        xpoints = [1,2,3]
        ypoints = [4,5,6]
        test_series = flot.Series(xpoints=xpoints, ypoints=ypoints)
        self.assertTrue(hasattr(test_series, '_x'))
        self.assertTrue(test_series._x.points, xpoints)
        self.assertTrue(hasattr(test_series, '_y'))
        self.assertTrue(test_series._y.points, ypoints)


class MyGraph(flot.Graph):
    "Graph Object"
    series1 = S1()
    series2 = S2()

    class Meta:
        grid = { 'clickable': True}


class GraphTest(unittest.TestCase):
    "Graph test class"

    def test_graph_builds_data(self):
        my_graph = MyGraph()
        graph_data_obj = json.loads(my_graph.json_data)
        x_graph_data1, y_graph_data1 = zip(*graph_data_obj[0]['data'])
        x_graph_data2, y_graph_data2 = zip(*graph_data_obj[1]['data'])
        self.assertEquals(list(x_graph_data1), S1()._x.points)
        self.assertEquals(list(y_graph_data1), S1()._y.points)
        self.assertEquals(list(x_graph_data2), S2()._x.points)
        self.assertEquals(list(y_graph_data2), S2()._y.points)

    def test_graph_receives_series_through_kwargs(self):
        sample_data = [SampleObject(x=i, y=i+10) for i in range(0, 10)]
        s3 = S3(data=sample_data)
        my_graph = flot.Graph(series1=S1(),
                                series2=S2(),
                                series3=s3)
        self.assertEquals(len(my_graph._series), 3)
        self.assertTrue(any([serie == S1() for serie in my_graph._series]))
        self.assertTrue(any([serie == S2() for serie in my_graph._series]))
        self.assertTrue(any([serie == s3 for serie in my_graph._series]))

    def test_graph_accept_multiple_type_series(self):
        x_points = [i for i in range(30, 40)]
        y_points = [i for i in range(40, 50)]
        x_field = flot.XVariable(points=x_points)
        y_field = flot.YVariable(points=y_points)
        series1 = flot.Series(x=x_field, y=y_field)
        my_graph = MyGraph(s1=series1)
        self.assertTrue(any([serie == series1 for serie in my_graph._series]))
        self.assertTrue(any([serie == S1() for serie in my_graph._series]))
        self.assertTrue(any([serie == S2() for serie in my_graph._series]))

    def test_graph_set_axis_mode(self):
        my_graph = flot.Graph(series1=S4())
        self.assertEquals(my_graph._options['xaxis']['mode'], 'time')

    def test_graph_get_options_on_Meta(self):
        my_graph = MyGraph()
        self.assertTrue('grid' in my_graph._options.keys())
        self.assertTrue(my_graph._options['grid']['clickable'])

    def test_graph_accepts_options(self):
        my_graph = flot.Graph(series1=S4(),
                options=flot.GraphOptions(xaxis={
                                                'mode': 'time',
                                                'format': '%d-%m-%y'
                                                }))
        self.assertEquals(my_graph._options['xaxis']['mode'], 'time')
        self.assertEquals(my_graph._options['xaxis']['format'], '%d-%m-%y')

    def test_graph_accepts_series_by_args(self):
        series1 = S1()
        series2 = S2()
        my_graph = flot.Graph([series1, series2])
        self.assertEquals(len(my_graph._series), 2)




from django.db import models

from django.contrib.auth.models import User

# Create your models here.
class Product(models.Model):
    ""
    title = models.CharField(max_lenght=128)


class StockRecord(models.Model):
    ""
    product = models.ForeignKey(Product)
    total_in_stock = models.IntegerField()


class StockCount(models.Model):
    "
    How many stock do we have for each product.
    total = StockRecord.objects.get(product=product).total_in_stock
    date = datetime.datetime.now()
    "
    product = models.ForeignKey(Product)
    total = models.IntegerField()
    date = models.DatetimeField()


class ProductCount(models.Model):
    """
    How many different products exists.
    total = Product.objects.count()
    date = datetime.datetime.now()
    """
    total = models.IntegerField()
    date = models.DatetimeField()




import flot
from sample.store.models import ProductCount

class ProductCount(flot.Series):
    ""
    total = flot.YVariable()
    date = flot.TimeXVariable()
    data = ProductCount.objects.all()

    class Meta:
        label = 'Total Products'


class ProductStockCount(flot.Series):
    ""
    total = flot.YVariable()
    date = flot.TimeXVariable()

    class Meta:
        label = 'Number in Stock for product'



import flot
import math
import datetime
from django.views.generic import TemplateView

class HomeView(TemplateView):
    template_name = 'home.html'

    def get_context_data(self, **kwargs):
        xy10 = flot.Series(x=flot.XVariable(points=range(1, 10)),
                           y=flot.YVariable(points=range(1, 10)),
                           options=flot.SeriesOptions(bars={'show': True},
                                                      label='y = 10*x'))

        xy20 = flot.Series(x=flot.XVariable(points=[i for i in range(1, 10)]),
                           y=flot.YVariable(points=[i*2 for i in range(1, 10)]),
                           options=flot.SeriesOptions(bars={'show': True},
                                                    label='y = 20*x',
                                                    color='green'))

        x_time_points = [datetime.date(2011, 1, i) for i in range(1, 20)]
        y_points = [float(1)/i for i in range(1, 20)]
        time1 = flot.Series(x=flot.TimeXVariable(points=x_time_points),
                            y=flot.YVariable(points=y_points),
                            options=flot.SeriesOptions(points={'show': True},
                                                        lines={'show': True},
                                                        label='y = 1/x',
                                                        color='blue'))

        graph_option = flot.GraphOptions(xaxis={'format': '%d/%m/%Y'})

        xpoints = map(math.radians ,range(1, 360))
        ypoints = map(math.sin, xpoints)
        sin_series = flot.Series(data=zip(xpoints, ypoints),
                            options=flot.SeriesOptions(label='sin(x)',
                                                       color='red'))

        last_series = flot.Series(xpoints=range(0, 10), ypoints=range(0, 10),
                                    options=flot.SeriesOptions(label='y = x'))

        inline_series = flot.Series(data=[(x*2, x) for x in range(0, 10)])

        context = {
                    'graph1': flot.Graph(series1=xy10, series2=xy20),
                    'graph2': flot.Graph(series1=time1, options=graph_option),
                    'sin_graph': flot.Graph(sin_series=sin_series),
                    'last_series': flot.Graph(last_series=last_series),
                    'all_series_graph': flot.Graph([xy10, xy20, last_series]),
                    'inline_series': flot.Graph([inline_series,])
                }
        return context



"""
pyflot demo app based on Flask
"""

import flot
from flask import Flask, render_template

class Fx(flot.Series):
    data = [(1, 3), (2, 2), (3, 5), (4, 4)]

class MyGraph(flot.Graph):
    fx = Fx()

app = Flask(__name__)

@app.route("/")
def root():
    my_graph = MyGraph()
    return render_template('index.html', my_graph=my_graph)

if __name__ == "__main__":
    app.run(debug=True)




from setuptools import setup

setup(name='PyFlot',
    version='0.2',
    description='Python Interface the JavaScript Flot chart librabry',
    long_description=open('README.rst', 'r').read(),
    license='MIT',
    author='Andre da Palma',
    author_email='andrefsp@gmail.com',
    url='http://github.com/andrefsp/pyflot',
    packages=['flot', 'flot.templatetags'],
    package_dir={'flot': 'flot'},
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: JavaScript",
        "Operating System :: OS Independent",
        "Topic :: Internet",
        "Topic :: Utilities",
        "Topic :: Software Development :: Libraries"
    ]
)



from django.conf.urls.defaults import *

import views
# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

urlpatterns = patterns('',
    # Example:
    # (r'^sample/', include('sample.foo.urls')),

    # Uncomment the admin/doc line below to enable admin documentation:
    # (r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    # (r'^admin/', include(admin.site.urls)),
    (r'^$', views.HomeView.as_view()),

    (r'^static/(?P<path>.*)$', 'django.views.static.serve', {'document_root': './static/'}),
)



from random import choice
from django import template

register = template.Library()

class GraphRenderer(template.Node):
    """
    This template tag is mean to render the graph object on the template
    """
    def __init__(self, graph, attr_string, dom_id):
        self.graph = template.Variable(graph)
        self.attr_string = attr_string
        self.dom_id = dom_id

    def render(self, context):
        graph = self.graph.resolve(context)
        ctx = {
            'dom_id': self.dom_id,
            'attr_string': self.attr_string,
            'graph_data': graph.json_data,
            'graph_options': graph.options
        }
        return """
        <div %(attr_string)s></div>
        <script type='text/javascript'>
            $.plot($("#%(dom_id)s"), %(graph_data)s, %(graph_options)s);
        </script>
        """ % ctx

@register.tag
def plot(parser, token):
    """
    Tag to plot graphs into the template
    """

    tokens = token.split_contents()
    tokens.pop(0)
    graph = tokens.pop(0)

    attrs = dict([token.split("=") for token in tokens])

    if 'id' not in attrs.keys():
        attrs['id'] = ''.join([chr(choice(range(65, 90))) for i in range(0, 5)])
    else:
        attrs['id'] = attrs['id'][1:len(attrs['id'])-1]

    attr_string = ''.join([" %s=%s" % (k, v) for k, v in attrs.iteritems()])
    return GraphRenderer(graph, attr_string, attrs['id'])
